import React__default, { useState, useRef, useEffect, forwardRef, useImperativeHandle, createElement } from 'react';
import { WidthProvider, Responsive } from 'react-grid-layout';
import { Button } from 'primereact/button';
import { OverlayPanel } from 'primereact/overlaypanel';
import { ProgressSpinner } from 'primereact/progressspinner';
import axios from 'axios';
import { Tooltip } from 'primereact/tooltip';
import ReactFC from 'react-fusioncharts';
import FusionCharts from 'fusioncharts';
import Charts from 'fusioncharts/fusioncharts.charts';
import FusionTheme from 'fusioncharts/themes/fusioncharts.theme.fusion';
import useResizeObserver from 'use-resize-observer';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;
  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
  return arr2;
}
function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);
  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }
  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var Session = function Session() {};

var Tools = /*#__PURE__*/function () {
  function Tools() {}
  Tools.GetLogitudeURL = function GetLogitudeURL() {
    var logitude_url = location.href.replace('index.html', '');
    if (location.href.indexOf('localhost') > -1) {
      logitude_url = 'http://localhost:9996';
    } else {
      var urlArr = location.href.split("/index.html");
      var url = urlArr[0];
      url = url.replace(url.substring(url.lastIndexOf('/'), url.length), "");
      logitude_url = url;
    }
    return logitude_url;
  };
  Tools.Map = function Map(from, to) {
    if (!from) return;
    if (!to) to = {};
    var keys = Object.keys(from);
    for (var _i = 0, _keys = keys; _i < _keys.length; _i++) {
      var k = _keys[_i];
      if (from[k]) {
        to[k] = from[k];
      }
    }
  };
  return Tools;
}();

var ApiServices = /*#__PURE__*/function () {
  function ApiServices() {
    this._apiUrl = Tools.GetLogitudeURL();
  }
  var _proto = ApiServices.prototype;
  _proto.getByFilters = function getByFilters(api, filters) {
    var token = this.getTokenFromStorage();
    var apiURL = "" + this._apiUrl + api;
    apiURL = this.ConcatFiltersToURL(filters, apiURL);
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };
  _proto.get = function get(api) {
    var token = this.getTokenFromStorage();
    var apiURL = "" + this._apiUrl + api;
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };
  _proto.post = function post(api, date) {
    var token = this.getTokenFromStorage();
    var apiURL = "" + this._apiUrl + api;
    return axios.post(apiURL, date, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };
  _proto.getTokenFromStorage = function getTokenFromStorage() {
    return Session.Token || '';
  };
  _proto.ConcatFiltersToURL = function ConcatFiltersToURL(filters, urlparameters) {
    var mykeys = Object.keys(filters);
    var addtionalFiltersValues = null;
    for (var i in mykeys) {
      var propName = mykeys[i];
      var propValue = filters[propName];
      var ignoreFilter = propName.indexOf("Operator") > 0 && propValue == "Equals" || propName == "AdditionalFilters";
      if (urlparameters != "?") {
        urlparameters = urlparameters.concat('&');
      }
      if (!ignoreFilter) {
        propValue = propValue ? encodeURIComponent(propValue) : '';
        urlparameters = urlparameters.concat(propName.concat('=').concat(propValue));
      }
      if (propName == "AdditionalFilters" && propValue.length > 0) {
        addtionalFiltersValues = JSON.stringify(propValue);
      }
    }
    if (addtionalFiltersValues) {
      urlparameters = urlparameters.concat("&AdditionalFilters=").concat(addtionalFiltersValues);
    }
    return urlparameters;
  };
  return ApiServices;
}();

var DashboardAnalyticsService = /*#__PURE__*/function () {
  function DashboardAnalyticsService() {
    this.controller = 'DashboardAnalytics';
    this._ApiServices = new ApiServices();
  }
  var _proto = DashboardAnalyticsService.prototype;
  _proto.getData = function getData(widget) {
    return this._ApiServices.post("/api/" + this.controller + "/PostGetDataAnalytic", widget);
  };
  _proto.getKpiData = function getKpiData(widget) {
    return this._ApiServices.post("/api/" + this.controller + "/PostGetKpiData", widget);
  };
  return DashboardAnalyticsService;
}();

var KpiChart = function KpiChart(props) {
  var _widget$current3;
  var _useState = useState(false),
    dataLoaded = _useState[0],
    setDataLoaded = _useState[1];
  var _useState2 = useState(),
    error = _useState2[0],
    setError = _useState2[1];
  var widget = useRef();
  var _useState3 = useState(),
    data = _useState3[0],
    setData = _useState3[1];
  useEffect(function () {
    widget.current = props.customChartProps.widget;
    setDataLoaded(false);
    setTimeout(function () {
      RefreshData();
    }, 500);
  }, []);
  var RefreshData = function RefreshData() {
    if (Session.Tenant == 0) {
      var emptyData = {};
      emptyData.Value = 0;
      setData(emptyData);
      setDataLoaded(true);
      return;
    }
    var dashboardAnalyticsService = new DashboardAnalyticsService();
    var postWidget = _extends({}, widget.current);
    postWidget.onChange = undefined;
    dashboardAnalyticsService.getKpiData(postWidget).then(function (result) {
      setData(result.data);
      setDataLoaded(true);
    }, function (error) {
      var _error$response, _error$response$data, _error$response2, _error$response2$data;
      console.log('error', error);
      if (error !== null && error !== void 0 && (_error$response = error.response) !== null && _error$response !== void 0 && (_error$response$data = _error$response.data) !== null && _error$response$data !== void 0 && _error$response$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response2 = error.response) === null || _error$response2 === void 0 ? void 0 : (_error$response2$data = _error$response2.data) === null || _error$response2$data === void 0 ? void 0 : _error$response2$data.ErrorMessage);else setError(error.message);
      setDataLoaded(true);
    });
  };
  var kpiClick = function kpiClick() {
    var _widget$current, _widget$current2, _widget$current2$Widg;
    var measureFieldId;
    if (widget !== null && widget !== void 0 && (_widget$current = widget.current) !== null && _widget$current !== void 0 && _widget$current.WidgetMeasures) measureFieldId = widget === null || widget === void 0 ? void 0 : (_widget$current2 = widget.current) === null || _widget$current2 === void 0 ? void 0 : (_widget$current2$Widg = _widget$current2.WidgetMeasures[0]) === null || _widget$current2$Widg === void 0 ? void 0 : _widget$current2$Widg.MeasureFieldId;
    props.customChartProps.onSelectDataPoint({
      MeasureFieldId: measureFieldId,
      Widget: widget.current
    });
  };
  function getRatioComponent() {
    if (!(data !== null && data !== void 0 && data.CompareWithPrevious)) return null;
    return React__default.createElement("div", {
      className: "dl-kpi-ratio-container"
    }, getArrowComponent(), React__default.createElement("div", {
      className: 'dl-kpi-ratio-text'
    }, data === null || data === void 0 ? void 0 : data.Ratio, " %"));
  }
  function getArrowComponent() {
    if (!data) return null;
    if ((data === null || data === void 0 ? void 0 : data.Ratio) == 0 || data.Value == data.ComparisonValue || (data.Value == "0" || data.Value == 0) && (data.ComparisonValue == "0" || data.ComparisonValue == 0)) return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-up-green"
    }), React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-down-red",
      style: {
        marginTop: "2px"
      }
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) > 1 && (data === null || data === void 0 ? void 0 : data.Increase) == "Positive") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-up-green"
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) > 1 && (data === null || data === void 0 ? void 0 : data.Increase) == "Negative") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-up-red"
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) < 1 && (data === null || data === void 0 ? void 0 : data.Increase) == "Positive") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-down-red"
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) < 1 && (data === null || data === void 0 ? void 0 : data.Increase) == "Negative") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-down-green"
    }));
    return null;
  }
  function getPreviousValueComponent() {
    if (!(data !== null && data !== void 0 && data.CompareWithPrevious)) return null;
    var prevtext = !(data !== null && data !== void 0 && data.ComparisonValue) ? '' : data === null || data === void 0 ? void 0 : data.ComparisonValue;
    prevtext = "Previous: " + prevtext;
    return React__default.createElement("div", {
      className: "dl-kpi-prev-container"
    }, React__default.createElement("div", {
      className: "dl-kpi-prev-text"
    }, prevtext), (data === null || data === void 0 ? void 0 : data.ComparisonAbbreviationSymbol) && React__default.createElement("div", {
      className: "dl-kpi-prev-abbreviationSymbol"
    }, data === null || data === void 0 ? void 0 : data.ComparisonAbbreviationSymbol));
  }
  function getValueData() {
    if ((data === null || data === void 0 ? void 0 : data.Value) == null) return React__default.createElement("div", {
      className: "dl-kpi-no-data"
    }, "No data to display.");
    return React__default.createElement("div", {
      className: "dl-kpi-value dl-kpi-textTrimming"
    }, data === null || data === void 0 ? void 0 : data.Value);
  }
  function getValueComponent() {
    return React__default.createElement("div", {
      className: "dl-kpi-sub-container"
    }, getValueData(), (data === null || data === void 0 ? void 0 : data.AbbreviationSymbol) && React__default.createElement("div", {
      className: "dl-kpi-abbreviationSymbol"
    }, data === null || data === void 0 ? void 0 : data.AbbreviationSymbol), (data === null || data === void 0 ? void 0 : data.Unit) && React__default.createElement("div", {
      className: "dl-kpi-unit"
    }, data === null || data === void 0 ? void 0 : data.Unit), getRatioComponent());
  }
  return !dataLoaded ? React__default.createElement("div", {
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : !error ? React__default.createElement("div", {
    onClick: kpiClick,
    className: ((_widget$current3 = widget.current) === null || _widget$current3 === void 0 ? void 0 : _widget$current3.Alighnment) == "Left" ? "dl-kpi-container-click" : "dl-kpi-container-click dl-kpi-center-container"
  }, React__default.createElement("div", {
    className: "dl-kpi-container"
  }, getValueComponent(), getPreviousValueComponent())) : React__default.createElement("div", {
    className: "dl-flex-content-center dl-full-hight",
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-tooltip"
  }, React__default.createElement("img", {
    width: 50,
    src: "https://www.seekpng.com/png/full/334-3345964_error-icon-png.png"
  }), React__default.createElement("span", {
    className: "dl-tooltiptext"
  }, error)));
};

var CustomChart = function CustomChart(props) {
  var project = function project() {
    var _props$widget;
    switch ((_props$widget = props.widget) === null || _props$widget === void 0 ? void 0 : _props$widget.TypeCode) {
      case "kpi":
        return React__default.createElement(KpiChart, {
          customChartProps: props
        });
      default:
        return React__default.createElement("div", null, "Invalid Type");
    }
  };
  return React__default.createElement("div", {
    className: "dl-custom-chart-container"
  }, project());
};

var mainYellowColor = "#F6CF33";
var mainRedColor = "#EE5F77";
var mainGreenColor = "#4AC76F";
var mainTurquoiseColor = "#32C7C7";
var mainBlueColor = "#369CFB";
var mainPurpleColor = "#935BE0";
var mainOrangeColor = "#FAAD14";

function getSeriesPositionColor(position) {
  switch (position) {
    case 1:
      return mainBlueColor;
    case 2:
      return mainGreenColor;
    case 3:
      return mainYellowColor;
    case 4:
      return mainTurquoiseColor;
    case 5:
      return mainPurpleColor;
    case 6:
      return mainOrangeColor;
    case 7:
      return mainRedColor;
    default:
      return null;
  }
}

function buildToolTip(widget, value, seriesName, label, index) {
  var padding = widget.TypeCode == 'line' && index > 0 ? 'padding-top: 3px' : '';
  return "\n    <div style=\"min-width: 108px;" + padding + "\">\n        <div style=\"display: flex;align-items: center;margin-bottom: 6px;\">\n            <span style=\"margin: 0; height: 10px; width: 10px; background-color: #A4A4A4; border-radius: 50%; display: inline-block;\"></span>\n            <p style=\" margin: 0; margin-left: 4px;font-family: 'Manrope'; font-style: normal;font-weight: 400; font-size: 11px; line-height: 100%;  color: #323232;  flex: none; order: 1;flex-grow: 0;\"> " + getTooltiplabel(label) + "</p>\n        </div>\n        <hr style=\"margin: 0;border-top: 1px solid #F0F0F0;\">\n        <div style=\"display: flex;align-items: center;margin:0;margin-top: 6px;\">\n            <p style=\"margin: 0;font-family: 'Manrope';font-style: normal;font-weight: 400; font-size: 11px;line-height: 100%;color: #A4A4A4;flex: none;order: 0;flex-grow: 0;\"> " + getMeasureType(seriesName) + "</p>\n            <p style=\"margin: 0;margin-left: 6px;font-family: 'Manrope';font-style: normal;font-weight: 500;font-size: 11px;line-height: 100%;  color: #323232;flex: none; order: 1;flex-grow: 0;\">" + getTooltipValue(value) + "</p>\n            " + getTooltipPercentage(widget) + " </div>\n    </div>\n    ";
}
function getTooltipPercentage(widget) {
  if (widget.TypeCode != "donut" && widget.TypeCode != "pie") return "";
  return "\n    <p style=\"margin: 0;margin-left: 2px;font-family: 'Manrope';font-style: normal;font-weight: 500;font-size: 11px;line-height: 100%;  color: #A4A4A4;flex: none; order: 1;flex-grow: 0;\">($percentValue)</p>\n    ";
}
function getTooltiplabel(value) {
  if (value) return value;
  return '$label';
}
function getTooltipValue(value) {
  if (value) return value;
  return '$dataValue';
}
function getMeasureType(seriesName) {
  if (!seriesName) return "Count:";
  return seriesName.replace(/ .*/, '') + ":";
}

function getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget) {
  if (seriesMeasuresIsEmpty(seriesMeasures)) return null;
  handelNullLabels(seriesMeasures);
  return {
    chart: chartInfo,
    categories: buildCategories(seriesMeasures),
    dataset: buildDataSet(seriesMeasures, widget)
  };
}
function getStackedDataSource(chartInfo, seriesMeasures, widget) {
  if (seriesMeasuresIsEmpty(seriesMeasures)) return null;
  handelNullLabels(seriesMeasures);
  return {
    chart: chartInfo,
    categories: buildStackedCategories(seriesMeasures),
    dataset: buildStackedDataSet(seriesMeasures, widget)
  };
}
function getSimpleDataSource(chartInfo, seriesMeasures, widget) {
  if (seriesMeasuresIsEmpty(seriesMeasures)) return null;
  handelNullLabels(seriesMeasures);
  return {
    chart: chartInfo,
    data: buildDataSource(seriesMeasures, widget)
  };
}
function handelNullLabels(seriesMeasures) {
  seriesMeasures.forEach(function (element) {
    element.Values.forEach(function (value) {
      if (!value.Label || value.Label == '') value.Label = 'Not defined';
    });
  });
}
function seriesMeasuresIsEmpty(seriesMeasures) {
  if (!seriesMeasures || seriesMeasures.length == 0) return true;
  var hasValue = false;
  seriesMeasures.forEach(function (item) {
    if (item.Values && item.Values.length > 0) hasValue = true;
  });
  return !hasValue;
}
function buildDataSource(values, widget) {
  var data = [];
  if (!values || !values[0]) return data;
  var series = values[0];
  data = series.Values.map(function (e, index) {
    var item = {
      value: e.Value,
      label: e.Label,
      id: {
        GroupById: e.GroupById,
        MeasureFieldId: series.MeasureFieldId
      },
      tooltext: buildToolTip(widget, null, series.Name, e.Label, 0),
      color: getSeriesPositionColor(index + 1)
    };
    return item;
  });
  return data;
}
function buildDataSet(seriesMeasures, widget) {
  var dataSet = [];
  var seriesPosition = 1;
  seriesMeasures.forEach(function (seriesMeasure) {
    var datas = seriesMeasure.Values.map(function (e, index) {
      return {
        value: e.Value,
        id: {
          GroupById: e.GroupById,
          MeasureFieldId: seriesMeasure.MeasureFieldId
        },
        tooltext: buildToolTip(widget, e.Value, seriesMeasure.Name, null, index)
      };
    });
    var color = getSeriesPositionColor(seriesPosition);
    dataSet.push({
      data: datas,
      seriesname: seriesMeasure.Name,
      color: color,
      anchorBgColor: color,
      anchorBorderColor: "#FFFFFF",
      anchorBorderThickness: "2",
      anchorHoverColor: color,
      renderAs: seriesMeasure.RenderAs == "default" ? null : seriesMeasure.RenderAs
    });
    seriesPosition++;
  });
  return dataSet;
}
function buildStackedDataSet(seriesMeasures, widget) {
  var dataSet = [];
  var seriesPosition = 1;
  var series = seriesMeasures[0];
  var uniqueGroups = [];
  series.Values.forEach(function (element) {
    if (!uniqueGroups.find(function (x) {
      return x.code == element.GroupByIdSec;
    })) uniqueGroups.push({
      code: element.GroupByIdSec,
      label: element.LabelSec
    });
  });
  uniqueGroups.forEach(function (group) {
    var color = getSeriesPositionColor(seriesPosition);
    dataSet.push({
      seriesName: group.label,
      color: color,
      data: buildStackedGroupData(group, series, widget)
    });
    seriesPosition++;
  });
  return dataSet;
}
function buildStackedGroupData(secGroup, series, widget) {
  var uniqueGroups = getUniqueGroupOrderdList(series);
  var data = [];
  uniqueGroups.forEach(function (group) {
    var _series$Values$find;
    data.push({
      value: (_series$Values$find = series.Values.find(function (x) {
        return x.GroupById == group.code && x.GroupByIdSec == secGroup.code;
      })) === null || _series$Values$find === void 0 ? void 0 : _series$Values$find.Value,
      id: {
        GroupById: group.code,
        MeasureFieldId: series.MeasureFieldId,
        GroupByIdSec: secGroup.code
      },
      tooltext: buildToolTip(widget, null, series.Name, group.label + " - " + secGroup.label, 0)
    });
  });
  return data;
}
function buildCategories(seriesMeasures) {
  var categories = seriesMeasures[0].Values.map(function (e) {
    return {
      label: e.Label
    };
  });
  var result = [];
  result.push({
    category: categories
  });
  return result;
}
function buildStackedCategories(seriesMeasures, widget) {
  var uniqueGroups = getUniqueGroupOrderdList(seriesMeasures[0]);
  var categories = uniqueGroups.map(function (e) {
    return {
      label: e.label
    };
  });
  var result = [];
  result.push({
    category: categories
  });
  return result;
}
function getUniqueGroupOrderdList(series) {
  var uniqueGroups = [];
  series.Values.forEach(function (element) {
    if (!uniqueGroups.find(function (x) {
      return x.code == element.GroupById;
    })) uniqueGroups.push({
      code: element.GroupById,
      label: element.Label
    });
  });
  return uniqueGroups;
}

var defautFontSize = "10";
var defautFont = "Manrope";
var defautColor = "#323232";
function BuildFusionChartObject(seriesMeasures, widget) {
  switch (widget === null || widget === void 0 ? void 0 : widget.TypeCode) {
    case 'bar':
      if (!widget.SecondaryGroupById) return getBarObject(seriesMeasures, widget);
      return getStackedBarObject(seriesMeasures, widget);
    case 'line':
      return getLineObject(seriesMeasures, widget);
    case 'pie':
      return getPieObject(seriesMeasures, widget);
    case 'donut':
      return getDonutObject(seriesMeasures, widget);
    case 'column':
      if (!widget.SecondaryGroupById) return getColumnObject(seriesMeasures, widget);
      return getClusteredColumnObject(seriesMeasures, widget);
    default:
      return {};
  }
}
function getBarObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "scrollbar2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = "1";
  chartInfo.showLegend = (seriesMeasures === null || seriesMeasures === void 0 ? void 0 : seriesMeasures.length) > 1 ? "1" : "0";
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chart.dataSource = getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function getStackedBarObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "scrollstackedbar2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = "1";
  chartInfo.showLegend = (seriesMeasures === null || seriesMeasures === void 0 ? void 0 : seriesMeasures.length) > 1 ? "1" : "0";
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chartInfo.showLegend = "1";
  chart.dataSource = getStackedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function getClusteredColumnObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "mscolumn2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = "1";
  chartInfo.showLegend = (seriesMeasures === null || seriesMeasures === void 0 ? void 0 : seriesMeasures.length) > 1 ? "1" : "0";
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chartInfo.showLegend = "1";
  chart.dataSource = getStackedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function getLineObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "scrollline2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = "1";
  chartInfo.showLegend = (seriesMeasures === null || seriesMeasures === void 0 ? void 0 : seriesMeasures.length) > 1 ? "1" : "0";
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "20";
  chartInfo.chartBottomMargin = "10";
  chartInfo.chartLeftMargin = "20";
  chartInfo.chartRightMargin = "50";
  chartInfo.anchorradius = "5";
  chartInfo.canvasBottomPadding = "10";
  chartInfo.canvasTopPadding = "10";
  chart.dataSource = getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function getDonutObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "doughnut2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.labelFontSize = defautFontSize;
  chartInfo.labelFont = defautFont;
  chartInfo.manageLabelOverflow = "1";
  chartInfo.useEllipsesWhenOverflow = "1";
  chartInfo.enableSmartLabels = "1";
  setPieDonutLegends(chartInfo, widget);
  chart.dataSource = getSimpleDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function getPieObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "pie2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.labelFontSize = defautFontSize;
  chartInfo.labelFont = defautFont;
  chartInfo.manageLabelOverflow = "1";
  chartInfo.useEllipsesWhenOverflow = "1";
  chartInfo.enableSmartLabels = "1";
  setPieDonutLegends(chartInfo, widget);
  chart.dataSource = getSimpleDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function getColumnObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "scrollcombidy2d";
  var chartInfo = getDefaultChartInfo();
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = "1";
  chartInfo.showLegend = (seriesMeasures === null || seriesMeasures === void 0 ? void 0 : seriesMeasures.length) > 1 ? "1" : "0";
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "20";
  chartInfo.chartBottomMargin = "10";
  chartInfo.chartLeftMargin = "20";
  chartInfo.chartRightMargin = "50";
  chartInfo.canvasBottomPadding = "10";
  setDualyAxis(chartInfo, widget, seriesMeasures);
  chart.dataSource = getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}
function setLineDash(chartInfo) {
  chartInfo.divLineDashed = "1";
  chartInfo.divLineDashLen = "5";
  chartInfo.divLineDashGap = "5";
  chartInfo.divLineColor = "#E7E7E7";
  chartInfo.divLineAlpha = "100";
  chartInfo.divLineThickness = "1";
}
function getDefaultChartobject() {
  var chart = {};
  chart.type = "column2d";
  chart.width = "100%";
  chart.height = "100%";
  chart.dataFormat = "json";
  return chart;
}
function getDefaultChartInfo(widget) {
  var chart = {};
  chart.theme = "fusion";
  chart.baseFont = defautFont;
  chart.baseFontSize = defautFontSize;
  chart.baseFontColor = defautColor;
  chart.legendCaptionFontSize = defautFontSize;
  chart.legendCaptionFontColor = defautColor;
  chart.legendCaptionFont = defautFont;
  chart.labelFontSize = defautFontSize;
  chart.labelFontColor = defautColor;
  chart.labelFont = defautFont;
  chart.legendItemFont = defautFont;
  chart.legendItemFontSize = defautFontSize;
  chart.sYAxisNameFont = defautFont;
  chart.sYAxisNameFontSize = defautFontSize;
  chart.sYAxisNameFontColor = defautColor;
  chart.pYAxisNameFont = defautFont;
  chart.pYAxisNameFontSize = defautFontSize;
  chart.pYAxisNameFontColor = defautColor;
  chart.xAxisNameFont = defautFont;
  chart.xAxisNameFontSize = defautFontSize;
  chart.xAxisNameFontColor = defautColor;
  chart.yAxisNameFont = defautFont;
  chart.yAxisNameFontSize = defautFontSize;
  chart.yAxisNameFontColor = defautColor;
  chart.enableSlicing = "0";
  chart.enableRotation = "0";
  chart.chartTopMargin = "10";
  chart.chartBottomMargin = "10";
  chart.chartLeftMargin = "40";
  chart.chartRightMargin = "40";
  chart.toolTipBgColor = "#FFFFFF";
  chart.showToolTipShadow = "1";
  chart.scrollHeight = "6";
  chart.scrollWidth = "6";
  chart.plotColorinTooltip = "0";
  return chart;
}
function setPieDonutLegends(chart, widget) {
  chart.showValues = "0";
  chart.showLabels = widget.LabelsPosition == 'InLegend' ? '0' : '1';
  chart.showLegend = widget.LabelsPosition == 'InLegend' ? '1' : '0';
  setOnChartLegend(chart, widget);
  setChartInLegend(chart, widget);
}
function setOnChartLegend(chart, widget) {
  if (widget.LabelsPosition != 'InLegend') return;
  chart.legendPosition = "right";
  chart.legendScrollBgColor = "#ffffff";
  chart.legendIconScale = 0.75;
  chart.scrollShowButtons = "0";
  chart.scrollShowButtons = "0";
  chart.legendCaptionAlignment = "right";
  chart.alignLegendWithCanvas = "1";
  chart.showPercentValues = "0";
  chart.chartRightMargin = "20";
}
function setChartInLegend(chart, widget) {
  if (widget.LabelsPosition == 'InLegend') return;
  chart.pieRadius = "70%";
}
function setDualyAxis(chartInfo, widget, seriesMeasures) {
  var _seriesMeasures$, _seriesMeasures$2;
  chartInfo.showSecondaryLimits = "0";
  chartInfo.showDivLineSecondaryValue = "0";
  if (!widget) return;
  if (!widget.WidgetMeasures) return;
  if (widget.WidgetMeasures.length < 2) return;
  if (widget.WidgetMeasures[1].YAxisType != 'dual') return;
  chartInfo.showSecondaryLimits = "1";
  chartInfo.showDivLineSecondaryValue = "1";
  chartInfo.pYAxisNameBorderPadding = 0;
  chartInfo.sYAxisNameBorderPadding = 0;
  var textLength = Math.max(seriesMeasures[0].Name.length, seriesMeasures[1].Name.length);
  chartInfo.pYAxisName = getTitle((_seriesMeasures$ = seriesMeasures[0]) === null || _seriesMeasures$ === void 0 ? void 0 : _seriesMeasures$.Name, textLength);
  chartInfo.sYAxisname = getTitle((_seriesMeasures$2 = seriesMeasures[1]) === null || _seriesMeasures$2 === void 0 ? void 0 : _seriesMeasures$2.Name, textLength);
}
function getTitle(text, maxLenght) {
  var textLenght = text.length;
  if (textLenght >= maxLenght) return text.replace(/\s+/g, '&nbsp;&nbsp;');
  var neededSpaces = Math.ceil((maxLenght - textLenght) / 2);
  var space = '&nbsp;';
  text = space.repeat(neededSpaces) + text + space.repeat(neededSpaces);
  return text.replace(/\s+/g, '&nbsp;&nbsp;');
}

var FusionChartHelper = /*#__PURE__*/function () {
  function FusionChartHelper(widget, chart) {
    this.chart = chart;
    this.widget = widget;
  }
  var _proto = FusionChartHelper.prototype;
  _proto.getPieRadius = function getPieRadius() {
    return this.pieRadius;
  };
  _proto.getLegendWidth = function getLegendWidth() {
    return this.legendWidth;
  };
  _proto.trimChartLabels = function trimChartLabels(chartWidth, chartHeight) {
    var _this$chart;
    if (!this.widget || this.widget.TypeCode != "pie" && this.widget.TypeCode != "donut") return;
    if (!this.chart) return;
    var dataSource = (_this$chart = this.chart) === null || _this$chart === void 0 ? void 0 : _this$chart.dataSource;
    if (!dataSource) return;
    var chartInfo = dataSource.chart;
    if (this.widget.LabelsPosition == 'InLegend') this.handleLegendWidth(chartWidth, chartHeight, chartInfo);else this.handlePieRadius(chartHeight, chartInfo);
  };
  _proto.handlePieRadius = function handlePieRadius(chartHeight, chartInfo) {
    var chartMinumunHeight = 236;
    var startPercantage = 70;
    var stepHeight = 60;
    var step = Math.floor((chartHeight - chartMinumunHeight) / stepHeight);
    if (step < 0) step = 0;
    var pieRadius = startPercantage - step * 10;
    if (pieRadius < 60) pieRadius = 60;
    if (window.innerWidth > 1700) {
      this.pieRadius = pieRadius;
      chartInfo.pieRadius = pieRadius + "%";
      return;
    }
    var reducedValue = Math.floor((1700 - window.innerWidth) / 100) + 1;
    if (reducedValue > 5) reducedValue = 5;
    pieRadius = pieRadius - reducedValue * 5;
    this.pieRadius = pieRadius;
    chartInfo.pieRadius = pieRadius + "%";
  };
  _proto.handleLegendWidth = function handleLegendWidth(chartWidth, chartHeight, chartInfo) {
    if (chartWidth < 730) chartInfo.legendWidth = "200";
    if (chartWidth >= 730) chartInfo.legendWidth = "400";
    if (chartWidth >= 1000) chartInfo.legendWidth = "500";
    this.legendWidth = chartInfo.legendWidth;
  };
  return FusionChartHelper;
}();

ReactFC.fcRoot(FusionCharts, Charts, FusionTheme);
FusionCharts.options['license']({
  key: 'prD4C-8eiA7A3A3C7E6G4B4A3J4C7B2D3D2nyqE1C3fd1npaE4D4tlA-21D7E4F4F1F1E1F4F1A10A8C2C5F5E2F2D1hwqD1B5D1aG4A19A32twbC6D3G4lhJ-7A9C11A5B-13ddA2I3A1B9B3D7A2B4G2H3H1F-7smC8B2XB4cetB8A7A5mxD3SG4F2tvgB2A3B2E4C3I3C7B3A4A3A2D3D2G4J-7==',
  creditLabel: false
});
var FusionChart = forwardRef(function (props, comRef) {
  var _useState = useState(false),
    dataLoaded = _useState[0],
    setDataLoaded = _useState[1];
  var _useState2 = useState(),
    error = _useState2[0],
    setError = _useState2[1];
  var widget = useRef();
  var _useState3 = useState({}),
    args = _useState3[0],
    setArgs = _useState3[1];
  var _useState4 = useState(),
    fChart = _useState4[0],
    setFchart = _useState4[1];
  var id = (Math.random() + 1).toString(36).substring(7);
  var progressRef = useRef(null);
  var fusionRef = useRef(null);
  var fusionChartHelper = useRef();
  useEffect(function () {
    widget.current = props.widget;
    setTimeout(function () {
      RefreshData();
    }, 500);
  }, []);
  useEffect(function () {
    var timeout;
    var handleResize = function handleResize() {
      clearTimeout(timeout);
      timeout = setTimeout(function () {
        HandleSizeChange();
      }, 200);
    };
    window.addEventListener("resize", handleResize);
    return function () {
      return window.removeEventListener("resize", handleResize);
    };
  }, []);
  useImperativeHandle(comRef, function () {
    return {
      onResizeFinish: function onResizeFinish() {
        HandleSizeChange();
      }
    };
  });
  var HandleSizeChange = function HandleSizeChange() {
    var _widget$current, _widget$current2;
    if (!widget || ((_widget$current = widget.current) === null || _widget$current === void 0 ? void 0 : _widget$current.TypeCode) != "pie" && ((_widget$current2 = widget.current) === null || _widget$current2 === void 0 ? void 0 : _widget$current2.TypeCode) != "donut") return;
    setTimeout(function () {
      var _fusionChartHelper$cu, _fusionChartHelper$cu2, _fusionChartHelper$cu3, _fusionRef$current, _fusionRef$current2, _widget$current3, _fusionChartHelper$cu4, _widget$current4, _fusionChartHelper$cu5;
      var oldPieRadius = fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu = fusionChartHelper.current) === null || _fusionChartHelper$cu === void 0 ? void 0 : _fusionChartHelper$cu.getPieRadius();
      var oldLegendWidth = fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu2 = fusionChartHelper.current) === null || _fusionChartHelper$cu2 === void 0 ? void 0 : _fusionChartHelper$cu2.getLegendWidth();
      fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu3 = fusionChartHelper.current) === null || _fusionChartHelper$cu3 === void 0 ? void 0 : _fusionChartHelper$cu3.trimChartLabels(fusionRef === null || fusionRef === void 0 ? void 0 : (_fusionRef$current = fusionRef.current) === null || _fusionRef$current === void 0 ? void 0 : _fusionRef$current.offsetWidth, fusionRef === null || fusionRef === void 0 ? void 0 : (_fusionRef$current2 = fusionRef.current) === null || _fusionRef$current2 === void 0 ? void 0 : _fusionRef$current2.offsetHeight);
      if (((_widget$current3 = widget.current) === null || _widget$current3 === void 0 ? void 0 : _widget$current3.LabelsPosition) != 'InLegend' && oldPieRadius == (fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu4 = fusionChartHelper.current) === null || _fusionChartHelper$cu4 === void 0 ? void 0 : _fusionChartHelper$cu4.getPieRadius())) return;
      if (((_widget$current4 = widget.current) === null || _widget$current4 === void 0 ? void 0 : _widget$current4.LabelsPosition) == 'InLegend' && oldLegendWidth == (fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu5 = fusionChartHelper.current) === null || _fusionChartHelper$cu5 === void 0 ? void 0 : _fusionChartHelper$cu5.getLegendWidth())) return;
      setDataLoaded(false);
      setTimeout(function () {
        setDataLoaded(true);
      }, 100);
    }, 100);
  };
  var RefreshData = function RefreshData() {
    if (Session.Tenant == 0) {
      handleData(null);
      return;
    }
    var dashboardAnalyticsService = new DashboardAnalyticsService();
    var postWidget = _extends({}, widget.current);
    postWidget.onChange = undefined;
    dashboardAnalyticsService.getData(postWidget).then(function (result) {
      handleData(result === null || result === void 0 ? void 0 : result.data);
    }, function (error) {
      var _error$response, _error$response$data, _error$response2, _error$response2$data;
      console.log('error', error);
      if (error !== null && error !== void 0 && (_error$response = error.response) !== null && _error$response !== void 0 && (_error$response$data = _error$response.data) !== null && _error$response$data !== void 0 && _error$response$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response2 = error.response) === null || _error$response2 === void 0 ? void 0 : (_error$response2$data = _error$response2.data) === null || _error$response2$data === void 0 ? void 0 : _error$response2$data.ErrorMessage);else setError(error.message);
      setDataLoaded(true);
    });
  };
  var handleData = function handleData(data) {
    var _progressRef$current, _progressRef$current2;
    var chart = BuildFusionChartObject(data, widget.current);
    fusionChartHelper.current = new FusionChartHelper(widget.current, chart);
    fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : fusionChartHelper.current.trimChartLabels(progressRef === null || progressRef === void 0 ? void 0 : (_progressRef$current = progressRef.current) === null || _progressRef$current === void 0 ? void 0 : _progressRef$current.offsetWidth, progressRef === null || progressRef === void 0 ? void 0 : (_progressRef$current2 = progressRef.current) === null || _progressRef$current2 === void 0 ? void 0 : _progressRef$current2.offsetHeight);
    setArgs(chart);
    setDataLoaded(true);
  };
  var dataplotClick = function dataplotClick(eventObj, dataObj) {
    props.onSelectDataPoint({
      GroupById: dataObj.id.groupbyid,
      MeasureFieldId: dataObj.id.measurefieldid,
      Widget: widget.current,
      GroupByIdSec: dataObj.id.groupbyidsec
    });
  };
  var renderComplete = function renderComplete(chart) {
    setFchart(chart);
  };
  var drawComplete = function drawComplete(event) {};
  var _useResizeObserver = useResizeObserver({
      onResize: function onResize(_ref) {
        var width = _ref.width,
          height = _ref.height;
        fChart === null || fChart === void 0 ? void 0 : fChart.resizeTo(width, height);
      }
    }),
    ref = _useResizeObserver.ref;
  return !dataLoaded ? React__default.createElement("div", {
    ref: progressRef,
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : !error ? React__default.createElement("div", {
    className: 'db-fc-parent-container',
    ref: fusionRef
  }, " ", React__default.createElement("div", {
    className: 'db-fc-container',
    ref: ref,
    id: id
  }, React__default.createElement(ReactFC, Object.assign({}, args, {
    "fcEvent-dataplotClick": dataplotClick,
    onRender: renderComplete,
    "fcEvent-drawComplete": drawComplete
  }))), " ") : React__default.createElement("div", {
    className: "dl-flex-content-center dl-full-hight",
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-tooltip"
  }, React__default.createElement("img", {
    width: 50,
    src: "https://www.seekpng.com/png/full/334-3345964_error-icon-png.png"
  }), React__default.createElement("span", {
    className: "dl-tooltiptext"
  }, error)));
});

var WidgetCard = forwardRef(function (props, comRef) {
  var _widget$current2, _widget$current3;
  var moreBtnToggle = useRef(null);
  var widget = useRef();
  var _useState = useState(false),
    isLoading = _useState[0],
    setIsLoading = _useState[1];
  var _useState2 = useState(false),
    showTooltip = _useState2[1];
  var fusionChartRef = useRef(null);
  var editWidget = function editWidget() {
    var _moreBtnToggle$curren;
    (_moreBtnToggle$curren = moreBtnToggle.current) === null || _moreBtnToggle$curren === void 0 ? void 0 : _moreBtnToggle$curren.hide();
    props.editBtnClicked(widget.current);
  };
  useEffect(function () {
    var _props$dataBinding;
    widget.current = props.widget;
    (_props$dataBinding = props.dataBinding) === null || _props$dataBinding === void 0 ? void 0 : _props$dataBinding.onEditWidget.subscribe(function (e) {
      var _widget$current;
      if ((e === null || e === void 0 ? void 0 : e.key) != ((_widget$current = widget.current) === null || _widget$current === void 0 ? void 0 : _widget$current.key)) return;
      setIsLoading(true);
      widget.current = _extends({}, e);
      setIsLoading(false);
    });
  }, []);
  useImperativeHandle(comRef, function () {
    return {
      onResizeFinish: function onResizeFinish() {
        var _fusionChartRef$curre;
        fusionChartRef === null || fusionChartRef === void 0 ? void 0 : (_fusionChartRef$curre = fusionChartRef.current) === null || _fusionChartRef$curre === void 0 ? void 0 : _fusionChartRef$curre.onResizeFinish();
      }
    };
  });
  return React__default.createElement("div", {
    className: "dl-panel",
    key: props.el.i,
    style: {
      cursor: props.isInEditMode ? 'move' : 'default'
    }
  }, React__default.createElement("header", null, React__default.createElement("div", {
    style: {
      display: 'flex',
      width: 'calc(100% - 3rem)'
    }
  }, React__default.createElement(Tooltip, {
    target: ".h-tool-tip"
  }), React__default.createElement("h1", {
    "data-pr-tooltip": (_widget$current2 = widget.current) === null || _widget$current2 === void 0 ? void 0 : _widget$current2.Title,
    "data-pr-position": "top",
    className: "h-tool-tip",
    onMouseEnter: function onMouseEnter() {
      return showTooltip(true);
    },
    onMouseLeave: function onMouseLeave() {
      showTooltip(false);
      setTimeout(function () {
        return showTooltip(true);
      }, 50);
    }
  }, (_widget$current3 = widget.current) === null || _widget$current3 === void 0 ? void 0 : _widget$current3.Title)), props.isInEditMode ? React__default.createElement(Button, {
    icon: "pi pi-ellipsis-v",
    className: "p-button-rounded p-button-text",
    onClick: function onClick(e) {
      var _moreBtnToggle$curren2;
      return (_moreBtnToggle$curren2 = moreBtnToggle.current) === null || _moreBtnToggle$curren2 === void 0 ? void 0 : _moreBtnToggle$curren2.toggle(e);
    },
    style: {
      color: "#292D30"
    }
  }) : null, React__default.createElement(OverlayPanel, {
    ref: moreBtnToggle,
    className: "buttons-overlay"
  }, React__default.createElement("div", {
    className: "buttons-list"
  }, React__default.createElement("div", {
    onClick: editWidget
  }, " Edit "), React__default.createElement("div", {
    onClick: function onClick() {
      var _props$widget;
      return props.deleteBtnClicked((_props$widget = props.widget) === null || _props$widget === void 0 ? void 0 : _props$widget.key);
    }
  }, "Remove")))), React__default.createElement("div", {
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-line"
  })), React__default.createElement("div", {
    className: "dl-dashboard-widget"
  }, " ", GetChartComponent(), " "));
  function GetChartComponent() {
    var _widget$current4, _widget$current5;
    if (isLoading || !((_widget$current4 = widget.current) !== null && _widget$current4 !== void 0 && _widget$current4.TypeCode)) return null;
    if (((_widget$current5 = widget.current) === null || _widget$current5 === void 0 ? void 0 : _widget$current5.TypeCode) == "kpi") return React__default.createElement(CustomChart, {
      dataBinding: props.dataBinding,
      widget: widget.current,
      widgetRef: props.widgetRef,
      onSelectDataPoint: props.onSelectDataPoint
    });
    return React__default.createElement(FusionChart, {
      ref: fusionChartRef,
      dataBinding: props.dataBinding,
      widget: widget.current,
      widgetRef: props.widgetRef,
      onSelectDataPoint: props.onSelectDataPoint
    });
  }
});

function deepClone(obj) {
  return JSON.parse(JSON.stringify(obj));
}

var ResponsiveReactGridLayout = WidthProvider(Responsive);
var DashboardDesigner = function DashboardDesigner(props) {
  var array12 = Array.from({
    length: 12
  }).fill(null);
  var minimumPlaceholderRowsCount = 12;
  var _useState = useState({
      lg: []
    }),
    layouts = _useState[0],
    setLayouts = _useState[1];
  var _useState2 = useState(Array.from({
      length: minimumPlaceholderRowsCount
    }).fill(null)),
    placeholderRowsArray = _useState2[0],
    setplaceholderRowsArray = _useState2[1];
  var _useState3 = useState(undefined),
    dateFilter = _useState3[0];
  var _useState4 = useState(false),
    isInEditMode = _useState4[0],
    setisInEditMode = _useState4[1];
  var widgetsDictionaryRef = useRef({});
  var _useState5 = useState({
      lg: []
    }),
    widgets = _useState5[0],
    setWidgets = _useState5[1];
  var gridLayoutComponent = useRef(null);
  var entered = useRef(false);
  var widgetCardRef = useRef([]);
  useEffect(function () {
    FillLayout(props.Widgetlayouts);
  }, [props.Widgetlayouts]);
  var FillLayout = function FillLayout(dashboardLayouts) {
    var clone = deepClone(dashboardLayouts);
    var layout = createLayout(clone);
    FillWidgetDictionary(clone);
    setWidgets(clone);
    setLayouts(layout);
    UpdatePlaceholderDimensions(layout);
  };
  var FillWidgetDictionary = function FillWidgetDictionary(clone) {
    for (var _iterator = _createForOfIteratorHelperLoose(clone.lg), _step; !(_step = _iterator()).done;) {
      var item = _step.value;
      widgetsDictionaryRef.current[item.key] = item;
    }
  };
  var createLayout = function createLayout(dashboardLayouts) {
    var layout = {
      lg: []
    };
    dashboardLayouts.lg.map(function (w) {
      return w.Layout;
    }).forEach(function (e) {
      layout.lg.push(e);
    });
    return layout;
  };
  useEffect(function () {
    setisInEditMode(props.editMode);
  }, [props.editMode]);
  function onLayoutChange(layout, changedLayouts) {
    UpdateWidget(changedLayouts);
    setLayouts(changedLayouts);
    UpdatePlaceholderDimensions(changedLayouts);
    if (entered.current == false) {
      entered.current = true;
      return;
    }
    props.onLayoutChange(changedLayouts);
  }
  function onResizeStop(event, oldItem) {
    var _widgetCardRef$curren, _widgetCardRef$curren2;
    widgetCardRef === null || widgetCardRef === void 0 ? void 0 : (_widgetCardRef$curren = widgetCardRef.current) === null || _widgetCardRef$curren === void 0 ? void 0 : (_widgetCardRef$curren2 = _widgetCardRef$curren.find(function (x) {
      return x.i == oldItem.i;
    })) === null || _widgetCardRef$curren2 === void 0 ? void 0 : _widgetCardRef$curren2.ref.onResizeFinish(widgetsDictionaryRef.current[oldItem.i].key);
  }
  function UpdateWidget(changedLayouts) {
    for (var _iterator2 = _createForOfIteratorHelperLoose(changedLayouts.lg), _step2; !(_step2 = _iterator2()).done;) {
      var item = _step2.value;
      if (widgetsDictionaryRef.current[item.i]) {
        widgetsDictionaryRef.current[item.i].Layout = item;
      }
    }
  }
  function deletePanel(id) {
    var newLayouts = {
      lg: layouts.lg.filter(function (e) {
        return e.i != id;
      })
    };
    setLayouts(newLayouts);
    var newWidgets = {
      lg: widgets.lg.filter(function (e) {
        return e.key != id;
      })
    };
    FillWidgetDictionary(newWidgets);
    setWidgets(newWidgets);
    props.onLayoutChange(newLayouts);
  }
  function editWidget(widget) {
    widget.ChangeSetOp = 'Update';
    props.openEditWidget(widget);
  }
  function UpdatePlaceholderDimensions(layouts) {
    if (!layouts) return;
    var max = minimumPlaceholderRowsCount;
    for (var _iterator3 = _createForOfIteratorHelperLoose(layouts['lg']), _step3; !(_step3 = _iterator3()).done;) {
      var layout = _step3.value;
      if (layout.y + layout.h > max) max = layout.y + layout.h;
    }
    var newPlaceholderRows = max + 2;
    newPlaceholderRows = newPlaceholderRows < minimumPlaceholderRowsCount ? minimumPlaceholderRowsCount : newPlaceholderRows;
    setplaceholderRowsArray(Array.from({
      length: newPlaceholderRows
    }).fill(null));
  }
  return React__default.createElement("div", {
    className: "dl-full-hight-scroll"
  }, React__default.createElement("div", {
    className: "dl-box"
  }, React__default.createElement("div", {
    className: "dl-row dl-content"
  }, React__default.createElement("div", {
    className: "dl-grid-container"
  }, React__default.createElement("div", {
    className: "dl-grid dl-grid--bordered"
  }, isInEditMode ? placeholderRowsArray.map(function (e, i) {
    return array12.map(function (e, i) {
      return React__default.createElement("div", {
        key: i,
        style: {
          height: layoutGridProps.rowHeight
        }
      });
    });
  }) : null), React__default.createElement(ResponsiveReactGridLayout, Object.assign({
    ref: gridLayoutComponent
  }, layoutGridProps, {
    layouts: layouts,
    compactType: "vertical",
    isDraggable: isInEditMode,
    isResizable: isInEditMode,
    onWidthChange: function onWidthChange() {},
    onResizeStop: onResizeStop,
    onLayoutChange: onLayoutChange
  }), widgets.lg.map(function (widget, i) {
    var _widget$Layout, _widget$Layout2;
    return React__default.createElement("div", {
      key: (_widget$Layout = widget.Layout) === null || _widget$Layout === void 0 ? void 0 : _widget$Layout.i
    }, React__default.createElement(WidgetCard, {
      isInEditMode: isInEditMode,
      key: (_widget$Layout2 = widget.Layout) === null || _widget$Layout2 === void 0 ? void 0 : _widget$Layout2.i,
      widget: widget,
      widgetRef: widgetsDictionaryRef.current[widget.key],
      el: widget,
      dataBinding: props.dataBinding,
      dateFilter: dateFilter,
      editBtnClicked: editWidget,
      deleteBtnClicked: deletePanel,
      onSelectDataPoint: props.onSelectDataPoint,
      ref: function ref(el) {
        var _widget$Layout3;
        widgetCardRef.current[i] = {
          ref: el,
          i: (_widget$Layout3 = widget.Layout) === null || _widget$Layout3 === void 0 ? void 0 : _widget$Layout3.i
        };
      }
    }));
  }))))));
};
var layoutGridProps = {
  rowHeight: 50,
  className: "layout",
  cols: {
    lg: 12,
    md: 6,
    sm: 2,
    xs: 1,
    xxs: 1
  },
  useCSSTransforms: true
};

var Dashboard = function Dashboard(props) {
  var _previewWidget$Layout, _previewWidget$Layout2;
  var Widgetlayouts = useRef({
    lg: []
  });
  var _useState = useState({
      lg: []
    }),
    widgetlayoutsState = _useState[0],
    setWidgetlayoutsState = _useState[1];
  var _useState2 = useState(false),
    isInEditMode = _useState2[0],
    setIsInEditMode = _useState2[1];
  var _useState3 = useState(props.isPreviewMode !== undefined ? props.isPreviewMode : false),
    isPreviewModeActive = _useState3[0],
    setIsPreviewModeActive = _useState3[1];
  var _useState4 = useState(props.widgetData),
    previewWidget = _useState4[0],
    setPreviewWidgett = _useState4[1];
  useEffect(function () {
    Session.CurrentUserId = props.userId;
    Session.Tenant = props.tenant;
    Session.Token = props === null || props === void 0 ? void 0 : props.token;
    setIsPreviewModeActive(props.isPreviewMode);
    props.dataBinding.onGetLayouts.subscribe(function (layouts) {
      Widgetlayouts.current = layouts;
      setWidgetlayoutsState(deepClone(Widgetlayouts.current));
    });
    props.dataBinding.isOnEditLayout.subscribe(function (e) {
      setIsInEditMode(e);
    });
    props.dataBinding.onAddWidget.subscribe(function (w) {
      Widgetlayouts.current.lg.push(w);
      setWidgetlayoutsState(deepClone(Widgetlayouts.current));
    });
  }, []);
  useEffect(function () {
    if (props.widgetData) {
      setPreviewWidgett(JSON.parse(JSON.stringify(props.widgetData)));
    }
  }, [props.widgetData]);
  var updateWidgets = function updateWidgets(layouts) {
    var layoutsDic = {};
    Object.keys(layouts).forEach(function (sizeType) {
      if (layouts[sizeType].length > 0) {
        layouts[sizeType].map(function (layout) {
          layoutsDic[layout.i] = layout;
        });
        return;
      }
    });
    var newWidgetlayouts = [];
    Widgetlayouts.current.lg.forEach(function (widget) {
      if (layoutsDic[widget.key]) {
        widget.StartPotistion = layoutsDic[widget.key].x + "," + layoutsDic[widget.key].y;
        widget.EndPosition = layoutsDic[widget.key].w + "," + layoutsDic[widget.key].h;
        widget.Layout = layoutsDic[widget.key];
        newWidgetlayouts.push(widget);
      }
    });
    Widgetlayouts.current = {
      lg: newWidgetlayouts
    };
    props.onChangeLayouts(Widgetlayouts.current);
  };
  return isPreviewModeActive && previewWidget ? createElement("div", {
    style: {
      width: "420px",
      height: "310px"
    }
  }, createElement("div", {
    style: {
      height: "100%"
    },
    key: "widget-" + (previewWidget === null || previewWidget === void 0 ? void 0 : (_previewWidget$Layout = previewWidget.Layout) === null || _previewWidget$Layout === void 0 ? void 0 : _previewWidget$Layout.i)
  }, createElement(WidgetCard, {
    isInEditMode: isInEditMode,
    key: previewWidget === null || previewWidget === void 0 ? void 0 : (_previewWidget$Layout2 = previewWidget.Layout) === null || _previewWidget$Layout2 === void 0 ? void 0 : _previewWidget$Layout2.i,
    widget: previewWidget,
    widgetRef: previewWidget,
    el: previewWidget,
    dataBinding: props.dataBinding,
    dateFilter: undefined,
    editBtnClicked: function editBtnClicked() {
      return true;
    },
    deleteBtnClicked: function deleteBtnClicked() {
      return true;
    },
    onSelectDataPoint: function onSelectDataPoint() {
      return true;
    }
  }))) : createElement("div", {
    className: 'dl-full-hight'
  }, createElement("div", {
    className: 'dl-dashboard-body'
  }, createElement(DashboardDesigner, {
    onSelectDataPoint: props.onSelectDataPoint,
    Widgetlayouts: widgetlayoutsState,
    dataBinding: props.dataBinding,
    editMode: isInEditMode,
    onLayoutChange: updateWidgets,
    openEditWidget: function openEditWidget(widget) {
      return props.openEditWidget(widget);
    }
  })));
};

export default Dashboard;
//# sourceMappingURL=index.modern.js.map
